/*
 Alina Campan - Cluster.java
 -----------------------------
 Date Created: 	03-19-2008
 Date Modified:	11-18-2006
 -----------------------------
 
 It contains the following class:
 	
	- Cluster - represents the elements of a cluster with their summary information;
*/

import java.util.*;
import java.io.*;

/** 
* A Cluster contains a set of elements (identified by their IDs in the DB) with their summary information. 
* A Cluster will always be formed by entities existing in the Db. 
*
* @author Alina Campan
*/
public class Cluster {

  HashSet elements; // cluster elements 
  ClusterSummary cs; // the summary of the cluster

  /** 
  * Constructs an empty Cluster.
  * @param d	the descriptor for all entities in the cluster.
  */
  Cluster(Descriptor d) {
    elements = new HashSet();
    cs = new ClusterSummary(d);
  }

  /** 
  * Constructs a new Cluster that contains the same elements as the cluster sent as parameter.
  * The new cluster will reference the same entities as the parameter cluster c.
  * @param c	the cluster to be copied.
  * @param d	the descriptor for all entities in the cluster.
  */
  Cluster(Cluster c, Descriptor d) {
  	elements = new HashSet(c.elements);
  	cs = new ClusterSummary(d);
// 	updateSummary(d); IMPLEMENT IT!!!
  }
  
  /** 
  * Constructs a new Cluster that contains the entities whose identifiers (names) 
  * are enumerated in the string line. 
  * @param d	the descriptor for all entities in the cluster.
  * @param line	a list of all entities identifiers to be included in this Cluster.
  */
  Cluster(Descriptor d, String line) {
    elements = new HashSet();
   	StringTokenizer st = new StringTokenizer(line, ", ");
  	
  	while (st.hasMoreTokens()) {
  		String eName = st.nextToken();
  		elements.add(Integer.parseInt(eName));
  	}

    cs = new ClusterSummary(d);
//  updateSummary(d); IMPLEMENT IT!!!
  }

  /**
   * Verifies if two cluster are equal (have the same entities in their composition)
   * @param cl the cluster which is compared with the current cluster
   * @return value 0 if the two clusters are equal or value 1 otherwise
   */
  public int equal(Cluster cl){
  	  if (this.elements.containsAll(cl.elements) && cl.elements.containsAll(this.elements))
  	  	  return 0;
  	  else
  	  	  return 1;
  }
  
  /** 
  * Removes all entities from the cluster.
  * @param d	the descriptor for all entities in the cluster.
  */
  public void empty(Descriptor d){
    elements.clear();
    cs = new ClusterSummary(d);
  }

  public void add(int idElem, Entity e) {
  	
  	if (size()==0) {
  		this.elements.add(idElem);
  		cs.updateSummaryFromEntity(e);
  	}
  	else {
	  	boolean flag = this.elements.add(idElem);
  	  	if (flag) 
  			cs = cs.updateSummaryWithNewEntity(e);
  	}
  }

  /** 
  * Returns the number of entities from the this cluster. 
  * @return		the size of the cluster.
  */
  public int size() {
  	return elements.size();
  }
  
  /** 
  * Displays to the standard output, on one line, the names of the entities that 
  * compose this cluster, separated by the ',' character.
  */  
  public void out() {
  	System.out.println(elements.toString());
  }

  public String toString() {
  	String result = "";
  	for (Object e: elements)
  		result += e.toString() + ",";
  	return result.substring(0, result.length()-1);
  }
  
  /** 
  * Writes to the PrintStream given as parameter, on one line, the names of the 
  * entities that compose this entity set, separated by the ',' character. Usually, 
  * the PrintStream is constructed for a file.
  * @param ps	the PrintStream object constructed for a file.
  */  
  public void outToPS(PrintStream ps) {
  	ps.println(elements.toString());
  }
  
  /** 
  * Returns the cluster summary of the this cluster.
  * @return		the cluster summary of the this cluster.
  */
  public ClusterSummary getSummary() {
  	return cs;
  }
  
  public Iterator getIterator(){
  	return elements.iterator();
  }
/*
  /** 
  * Computes the information loss for the this cluster.
  * For a cluster with k elements (r1, r2, .., rk) where the set of key attributes 
  * consists of m numerical attributes (N1, .., Nm) and n categorical attributes
  * (C1, ..., Cn), the information loss is defined as:
  * IL = Cluster_size * D.
  * D is the sum of information loss introduced by each attribute.
  * For a numerical attribute i, its term is:
  * = |MAX(Ni) - MIN(Ni)| / |Ni|, where MAX(Ni) and MIN(Ni) are the maximum and
  * minimim values of that attribute in the this cluster and |Ni| is the size of numeric domain Ni.
  * For a categorial attribute, its term is:
  *	= H(first common ancestor of the values of that attribute for all entities from cluster) / 
  *   H(hierarchy of the attribute), where and H denotes the function height of a 
  * hierarchy or subhierarchy.
  *
  * @param d		the descriptor for all entities in the cluster.
  * @param csAll	the summary for a cluster that contains all elemets from the dataset.
  * /
  public double InformationLoss (Descriptor d, ClusterSummary csAll) {
		double result = 0;
		
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			
			switch (a.getType()) {
      			case AttributeType.NUMERIC : 
      				result += (((Interval) cs.summ[i]).getMax()-((Interval) cs.summ[i]).getMin())/
							  (((Interval) csAll.summ[i]).getMax()-((Interval) csAll.summ[i]).getMin());
      				// System.out.print((((Interval) cs.summ[i]).getMax()-((Interval) cs.summ[i]).getMin())+"/" +
					//		  (((Interval) csAll.summ[i]).getMax()-((Interval) csAll.summ[i]).getMin())+ " + ");
   			  		break;
			    case AttributeType.CATEGORICAL :
			    	result += (a.getHierarchy().height(a.getHierarchy().searchNode((String) cs.summ[i], a.getHierarchy().getRoot())) + Double.parseDouble("0.0"))/
			    			  a.getHierarchy().height(a.getHierarchy().getRoot());
      				// System.out.print((a.getHierarchy().height(a.getHierarchy().searchNode((String) cs.summ[i], a.getHierarchy().getRoot())) + Double.parseDouble("0.0")) +"/"+
			    	//		  a.getHierarchy().height(a.getHierarchy().getRoot())+ " + ");
			    			  // the height of the subhierarchy having root the node containing summ[i] /
			    			  // the height of the entire hierarchy of the attribute
   	    		  	break;
    			}
		}
		
		return result * elements.size();
  }
	
    /** 
    * Returns 0 if e entity (e does not belong to the this cluster) can be generalized to this cluster.
    * In other words e is in the hypercube containing the cluster.
    * @param e		an entity not belonging to the this cluster.
    * @return		0 if e belong to this cluster.
    *				other value otherwise.
    * /
	// returns 0 if e can be generalized to current cluster (i.e. e is in the hypercube containing the cluster)
	public int coversClusterEntity(Entity e) {
		int result = 0;
		
		for (int i=0; (i<cs.d.getNoQuaAttr()) && (result==0); i++) {
			Attribute a = cs.d.getQuaAttribute(i);
			
			switch (a.getType()) {
      			case AttributeType.NUMERIC : 
      				result = ((Interval) cs.summ[i]).belong(((Double) e.getVal(i)).doubleValue());
   			  		break;
			    case AttributeType.CATEGORIAL :
			    	Hierarchy h = a.getHierarchy();
			    	if (h.searchNode((String) e.getVal(i), h.searchNode((String) cs.summ[i],h.getRoot())) == null)
			    		result = 1;
   	    		  	break;
    			}
			}
		return result;			
	}
*/	
} // class
