
/*
 Alina Campan - ClusterSummary.java
 -----------------------------
 Date Created: 	03-19-2008
 Date Modified:	03-19-2008
 -----------------------------
 
 It contains the following class:
 	
	- ClusterSummary - represents the summary of a cluster;
*/

/** 
* A ClusterSummary contains the summary of a cluster, in other words 
* information to which the cluster entities are to be generalized to
* The ClusterSummary has the following form:
* (m1,M1),(m2,M2),.., (mn, Mn), c1, c2, .., cm, where the first n elements are 
* intervals that describes the minimim and maximum value for the coresponding 
* numerical attribute for the current cluster and the last m elements are strings
* that represents the minimal generalization of categorical attributes.
*
* @author Alina Campan
*/
public class ClusterSummary {
	Object[] summ;
	Descriptor d;
	
	/** 
	* Constructs a summary that coresponds to an empty cluster.
	* @param desc	the descriptor for all entities in the cluster.
	*/
	ClusterSummary (Descriptor desc) {
		d = desc;
		summ = new Object[d.getNoQuaAttr()];
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : summ[i] = new Interval();
        		break;
			    case AttributeType.CATEGORICAL : summ[i] = new String("");
        		break;
    		}
		}
	}
	
	/** 
	* Constructs a summary that coresponds to the cluster c.
	* @param c	the cluster for which the summary will be constructed.
	*/
	ClusterSummary(ClusterSummary c) {
		d = c.d;
		
		summ = new Object[d.getNoQuaAttr()];
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : summ[i] = new Interval(((Interval) c.summ[i]).getMin(), ((Interval) c.summ[i]).getMax());
        		break;
			    case AttributeType.CATEGORICAL : summ[i] = new String((String) c.summ[i]);
        		break;
    		}
		}
	}
	
	/** 
	* Writes to the standard output the cluster summary.
	*/
	public void out() {
		System.out.println(toString());
	}


	/** 
	* Produces a String representation of the cluster summary.
	*/
	public String toString() {
		String result = "";
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : result += "'" + ((Interval) summ[i]).toString() + "', ";
        		break;
			    case AttributeType.CATEGORICAL : result += "'" + (String) summ[i] + "', ";
        		break;
    		}
		}
		return result.substring(0, result.length()-2);
	}

	/** 
	* Checks weather the ClusterSummary c overlaps this ClusterSummary.
	* For each numerical attribute the intersection of the corresponding intervals is not empty
	* For each categorical attribute, the coresponding summary values share at least one leaf 
	* in the value generalization tree. 
	* @param c	the ClusterSummary for which the overlap is checked.
	* @return 	0, if overlap
	*			1, otherwise   
	*/
	public int overlap (ClusterSummary c) {
		// 0 if summary c covers the current summary : 
		// - has intersection for every numeric attribute, 
		// - covers for every categorial attribute
		int result=0;
		for (int i=0; (i<d.getNoQuaAttr()) && (result==0); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : 
      				if (((Interval) summ[i]).intersect((Interval) c.summ[i]) != 0) result=1;
        			break;
			case AttributeType.CATEGORICAL : 
			    	Hierarchy h = a.getHierarchy();
			    	if (h.isAncestor(h.searchNode((String) c.summ[i],h.getRoot()), 
			    				   	 h.searchNode((String) summ[i],h.getRoot()))!=0) result=1;
        			break;
    		}
		}
		return result;
	}

	/** 
	 * Creates a summary for an entity. It is an initialization method used by updateSummary.
	 * Note: It should be private since it is used by updateSummary only. 
	 * @param e 	the entity used for initialization.
	 */
	 
	 public void updateSummaryFromEntity (Entity e) {
	 	for (int i=0; i<d.getNoQuaAttr(); i++) {
	 		Attribute a = d.getQuaAttribute(i);
	 		switch (a.getType()) {
	 			case AttributeType.NUMERIC : summ[i] = new Interval(((Double) e.getVal(i)).doubleValue());
	 			break;
	 			case AttributeType.CATEGORICAL : summ[i] = new String((String) e.getVal(i));
	 			break;
	 		}
	 	}
	 }
	 
	 public ClusterSummary updateSummaryWithNewEntity (Entity e) {
	 	ClusterSummary cs = new ClusterSummary(this);
	 	
	 	for (int i=0; i<d.getNoQuaAttr(); i++) {
	 		Attribute a = d.getQuaAttribute(i);
	 		switch (a.getType()) {
	 			case AttributeType.NUMERIC : 
	 				if (((Double) e.getVal(i)).doubleValue() < ((Interval) cs.summ[i]).getMin())
	 					((Interval) cs.summ[i]).setMin(((Double) e.getVal(i)).doubleValue());
	 				else if (((Double) e.getVal(i)).doubleValue() > ((Interval) cs.summ[i]).getMax())
	 					((Interval) cs.summ[i]).setMax(((Double) e.getVal(i)).doubleValue());
	 				break;
	 			case AttributeType.CATEGORICAL : 
   			  		HNode n1 = a.getHierarchy().searchNode((String) cs.summ[i], a.getHierarchy().getRoot());
   			  		HNode n2 = a.getHierarchy().searchNode((String) e.getVal(i), a.getHierarchy().getRoot());
   			  		if (n1==null) System.out.println("n1: " + (String) cs.summ[i] + " not found");
   			  		if (n2==null) System.out.println("n2: " + (String) e.getVal(i) + " not found - coming from entity " + e.getName());
		      		cs.summ[i] = (String) a.getHierarchy().findAncestor(n1,n2).getData();
	 				break;
	 		}
	 	}
	 	
	 	return cs;
	 }
	 
	 public double InformationLoss (ClusterSummary csAll) {
	 // IL of this cluster summary reported to csAll, translated to [0,1]
		double result = 0;
				
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			
			switch (a.getType()) {
      			case AttributeType.NUMERIC : 
      				result += (((Interval) summ[i]).getMax()-((Interval) summ[i]).getMin())/
							  (((Interval) csAll.summ[i]).getMax()-((Interval) csAll.summ[i]).getMin());
      				// System.out.print((((Interval) summ[i]).getMax()-((Interval) summ[i]).getMin())+"/" +
					//		  (((Interval) csAll.summ[i]).getMax()-((Interval) csAll.summ[i]).getMin())+ " + ");
   			  		break;
			    case AttributeType.CATEGORICAL :
			    	result += (a.getHierarchy().height(a.getHierarchy().searchNode((String) summ[i], a.getHierarchy().getRoot())) + Double.parseDouble("0.0"))/
			    			  a.getHierarchy().height(a.getHierarchy().getRoot());
      				// System.out.print((a.getHierarchy().height(a.getHierarchy().searchNode((String) summ[i], a.getHierarchy().getRoot())) + Double.parseDouble("0.0")) +"/"+
			    	//		  a.getHierarchy().height(a.getHierarchy().getRoot())+ " + ");
			    			  // the height of the subhierarchy having root the node containing summ[i] /
			    			  // the height of the entire hierarchy of the attribute
   	    		  	break;
    			}
		}
		
		return result / d.getNoQuaAttr();
  }
}
