/*
 Alina Campan - Descriptor.java
 -----------------------------
 Date Created: 	03-18-2008
 Date Modified:	03-18-2008
 -----------------------------
 
 It contains the following class:
 	
	- Descriptor - represents the relational schema that characterizes the data to be anonymized;
*/

/** 
* A descriptor describes the relational schema of the dataset for p-sensitive k-anonymity
* It contains a set of attributes. Each attribute is of type numerical or categorical. 
*
* @author Alina Campan 
*/

import java.util.*;
import java.io.*;


public class Descriptor {
      int noNumericQuaAttr; 	 // the number of numerical quasi-identifier attributes
	  int noCategoricalQuaAttr;   // the number of categorical quasi-identifier attributes
	  Attribute[] attrQua;       // the set of quasi-identifier attributes, from 0 to noNumericQuaAttr + 
	  							 //	noCategoricalQuaAttr - 1
	                             // the first noNumericQuaAttr are numerical attribute
	 
	  int noNumericSenAttr; 	 // the number of numerical sensitive attributes
	  int noCategoricalSenAttr;   // the number of categorical sensitive attributes
	  Attribute[] attrSen;       // the set of sensitive attributes, from 0 to noNumericSenAttr + 
		 						 //	noCategoricalSenAttr - 1
      							 // the first noNumericSenAttr are numerical attribute

	  /** 
	  * Constructs a descriptor. It takes a file name as parameter.
	  * The file must have the following structure:
	  *
	  *             format of the descriptor file:
	  *             no_numerical_quasi-identifier_attributes no_categorical_quasi-identifier_attributes no_numerical_sensitive_attributes no_categorical_sensitive_attributes
	  *             a_1_name a_2_name  . . . a_no_numerical_quasi-identifier_attributes_name
	  *             a_1_name a_2_name  . . . a_no_categorical_quasi-identifier_attributes_name
	  *             a_1_name a_2_name  . . . a_no_numerical_sensitive_attributes_name
	  *             a_1_name a_2_name  . . . a_no_categorical_sensitive_attributes_name
	  *
	  * Example:
	  *             3 2 2 1
	  *             nq1 nq2 nq3
	  *             cq4 cq5
	  *             ns1 ns2
	  *             cs1
	  *
	  * @param fileName	the file name.
	  */
	  Descriptor(String fileName) throws Exception {
		  BufferedReader    br = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
		  String line = br.readLine();
	      StringTokenizer st = new StringTokenizer(line, "\t ");
		  noNumericQuaAttr = Integer.parseInt(st.nextToken());
	      noCategoricalQuaAttr = Integer.parseInt(st.nextToken());
	      noNumericSenAttr = Integer.parseInt(st.nextToken());
	      noCategoricalSenAttr = Integer.parseInt(st.nextToken());

	      attrQua = new Attribute[noNumericQuaAttr + noCategoricalQuaAttr];
	      attrSen = new Attribute[noNumericSenAttr + noCategoricalSenAttr];

	      // construct the numeric quasi-identifier attributes
	      line = br.readLine();
	      st = new StringTokenizer(line, "\t ");
	      String name;
	      for (int i=0; i<noNumericQuaAttr; i++) {
	    	  name = st.nextToken();
	    	  attrQua[i] = new Attribute(name, AttributeType.NUMERIC);
	      }

	      // construct the categorical quasi-identifier attributes
	      line = br.readLine();
	      st = new StringTokenizer(line, "\t ");
	      for (int i=0; i<noCategoricalQuaAttr; i++) {
	    	  name = st.nextToken();
	    	  attrQua[noNumericQuaAttr+i] = new Attribute(name, AttributeType.CATEGORICAL);
	      }
	      
	      // construct the numeric sensitive attributes
	      line = br.readLine();
	      st = new StringTokenizer(line, "\t ");
	      for (int i=0; i<noNumericSenAttr; i++) {
	    	  name = st.nextToken();
	    	  attrSen[i] = new Attribute(name,AttributeType.NUMERIC,Double.parseDouble(st.nextToken()));
	      }

	      // construct the categorical sensitive attributes
//System.out.println("beginning constructing categorical sensitive attributes");
	      line = br.readLine();
	      st = new StringTokenizer(line, "\t ");
//System.out.println("beginning processing " + line);
	      for (int i=0; i<noCategoricalSenAttr; i++) {
	    	  name = st.nextToken();
//System.out.println("construct attr " + name);
	    	  attrSen[noNumericSenAttr+i] = new Attribute(name,AttributeType.CATEGORICAL,Double.parseDouble(st.nextToken()));
	     }
	    br.close();
	  }

	  /** 
	  * Returns the number of numerical quasi-identifier attributes
	  * @return	the number of numerical quasi-identifier attributes.
	  */  
	  public int getNoNumericQuaAttr() {
	  	return noNumericQuaAttr;
	  }

	  /** 
	  * Returns the number of categorical quasi-identifier attributes.
	  * @return	the number of categorical quasi-identifier attributes.
	  */  
	  public int getNoCategoricalQuaAttr() {
	  	return noCategoricalQuaAttr;
	  }
	  
	  /** 
	  * Returns the number of numerical sensitive attributes
	  * @return	the number of numerical sensitive attributes.
	  */  
	  public int getNoNumericSenAttr() {
		  return noNumericSenAttr;
	  }

	  /** 
	   * Returns the number of categorical sensitive attributes.
	   * @return	the number of categorical sensitive attributes.
	   */  
	  public int getNoCategoricalSenAttr() {
		  return noCategoricalSenAttr;
	  }

	  /** 
	  * Returns the total number of quasi-identifier attributes.
	  * @return	the total number of quasi-identifier attributes.
	  */  
	  public int getNoQuaAttr() {
	  	return (getNoNumericQuaAttr()+getNoCategoricalQuaAttr());
	  }
	  
	  /** 
	   * Returns the total number of sensitive attributes.
	   * @return	the total number of sensitive attributes.
	  */  
	  public int getNoSenAttr() {
		  return (getNoNumericSenAttr()+getNoCategoricalSenAttr());
	  }
	  
	  /** 
	  * Returns the quasi-identifier Attribute object located in the specified position.
	  * @param i	the position of the Attribute object.
	  * @return	the ith attribute.
	  */  
	  public Attribute getQuaAttribute(int i) {
	  	// i must be in the acceptable range 0, noNumericQuaAttr+noCategoricalQuaAttr-1;
	  	return attrQua[i];
	  }
	  
	  /** 
	   * Returns the sensitive Attribute object located in the specified position.
	   * @param i	the position of the Attribute object.
	   * @return	the ith attribute.
	   */  
	  public Attribute getSenAttribute(int i) {
		  // i must be in the acceptable range 0, noNumericSenAttr+noCategoricalSenAttr-1;
		  return attrSen[i];
	  }

	  /** 
	  * Displays to the standard output this Descriptor. Each attribute will be displayed on one line containing its name and type.
	  */  
	  public void out() {
	    for (int i=0; i<noNumericQuaAttr; i++) attrQua[i].out();
	    for (int i=0; i<noCategoricalQuaAttr; i++) attrQua[noNumericQuaAttr+i].out();
	    for (int i=0; i<noNumericSenAttr; i++) attrSen[i].out();
	    for (int i=0; i<noCategoricalSenAttr; i++) attrSen[noNumericSenAttr+i].out();
	  }

	/*
	// for testing purposes
	  public static void main(String args[]) throws Exception {
	    Descriptor d = new Descriptor("descriptor.txt");
	    d.out();
	  }
	*/
}
