/*
 Alina Campan - Entity.java
 -----------------------------
 Date Created: 	03-21-2008
 Date Modified:	03-21-2008
 -----------------------------
 
 It contains the following class:
 	
	- Entity - represents an entity (or record) from the dataset for p-sensitive k-anonymity ;
*/

/** 
* Entity represents a record or an object from the dataset for for p-sensitive k-anonymity . This data set has 
* associated a descriptor, which describes the common structure or schema of all the entities in 
* the data set. In other words, all the entities in the dataset have the same structure which is 
* described by a descriptor.
*
* An entity has a unique name between the set of all entities in the dataset for p-sensitive k-anonymity. 
* The entity name is memorized in the data field name.
* 
* An entity is composed from one value for each of the attributes that compose the descriptor 
* associated to the dataset. The value for an attribute is:
* - a number, if the corresponding attribute is of numeric type;
* - a value found in the attribute hierarchy, if the attribute is categorial; usually, in microdata, 
*   the categorial attribute values are from the leaves of the hierarchy.
* The attribute values are memorized in the data field val; val[i] is the value for the ith attribute 
* in the dataset descriptor. The value for a numeric attribute is a real number represented as double, 
* the value for a categorial attribute is a string represented as String.
* 
* @author Alina Campan 
*/

import java.util.*;
import java.sql.*;

public class Entity {

	String name; 	// object name
	Object[] val; 	// attribute values
	double[] w; 	// attribute weights

	Entity(Descriptor d, Connection con, String tableName, int ID) throws Exception {
		val = new Object[d.getNoNumericQuaAttr() + d.getNoCategoricalQuaAttr()+d.getNoNumericSenAttr() + d.getNoCategoricalSenAttr()]; 
		w = new double[d.getNoSenAttr()];

	    Statement stmt = con.createStatement();
		stmt.execute("select * from " + tableName + " where ID = " + ID);
		ResultSet rs = stmt.getResultSet();
		rs.next();
		
		name = rs.getInt("ID")+"";
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : val[i] = new Double(rs.getDouble(a.getName()));
        		break;
			    case AttributeType.CATEGORICAL : val[i] = rs.getString(a.getName());
        		break;
    		}
		}
		for (int i=0; i<d.getNoSenAttr(); i++) {
			Attribute a = d.getSenAttribute(i);
			w[i] = a.getW();
			switch (a.getType()) {
      			case AttributeType.NUMERIC : val[i+d.getNoQuaAttr()] = new Double(rs.getDouble(a.getName()));;
        		break;
			    case AttributeType.CATEGORICAL : val[i+d.getNoQuaAttr()] = rs.getString(a.getName());
        		break;
    		}
		}
	}
	
	/** 
	* Constructs an entity with the schema d, and which has the attributes values enumerated in s. 
	* The parameter s must conform to the schema specified by the descriptor d. String s also 
	* contains as first item the name of the entity. 
	* For example, let d be the descriptor that represents the schema (age N, country C, occupation C), 
	* where N stands for numeric and C for categorial. Then an entity conforming to this scheme is the 
	* one represented by the string s = "e3 40 canada teacher".
	*
	* @param d	the descriptor the new entity is conform to.
	* @param s	a string containing the name of the entity and the values of the attributes.
	*/
	Entity(Descriptor d, String s) {
		StringTokenizer st = new StringTokenizer(s, "\t ");
		val = new Object[d.getNoNumericQuaAttr() + d.getNoCategoricalQuaAttr()+d.getNoNumericSenAttr() + d.getNoCategoricalSenAttr()]; 
		w = new double[d.getNoSenAttr()];
		name = st.nextToken();
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			switch (a.getType()) {
      			case AttributeType.NUMERIC : val[i] = new Double(Double.parseDouble(st.nextToken()));
        		break;
			    case AttributeType.CATEGORICAL : val[i] = new String(st.nextToken());
        		break;
    		}
		}
		for (int i=0; i<d.getNoSenAttr(); i++) {
			Attribute a = d.getSenAttribute(i);
			w[i] = a.getW();
			switch (a.getType()) {
      			case AttributeType.NUMERIC : val[i+d.getNoQuaAttr()] = new Double(Double.parseDouble(st.nextToken()));
        		break;
			    case AttributeType.CATEGORICAL : val[i+d.getNoQuaAttr()] = new String(st.nextToken());
        		break;
    		}
			
		}
	}
	
	/**
	 * Verifies if two entities are equal (if they have the same name)
	 * @param e the entity which is compared with the current one
	 * @return value 0 if the two values are equal or value 1 otherwise
	 */
	public int equal(Entity e){
		if (((this.name).compareTo(e.getName())) == 0){ 
				return 0;
		}		
		else
			return 1;
	}
	
	/** 
	* Sets the name of the entity.
	* @param n	the name of the current entity.
	*/
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * 
	 * @return w[]
	 */
	public double[] getW() {
		return w;
	}
	
	/**
	 * 
	 * @param w
	 */	
	public void setW(double[] w) {
		this.w = w;
	}
	
	/** 
	* Gets the name of the entity.
	* @return	the name of the current entity.
	*/
	public String getName() {
		return name;
	}

	/**
	 * 
	 * @return
	 */
	public Object[] getVal() {
		return val;
	}
	/**
	 * 
	 * @param val
	 */
	public void setVal(Object[] val) {
		this.val = val;
	}
	/** 
	* Gets the ith attribute value for the current entity. Attribute values (elements in 
	* array val) are indexed beginning from 0.
	*
	* @param i	the index of the attribute for which the value in the current entity is requested.
	* @return	the ith attribute value for the current entity.
	*/
	public Object getVal(int i) {
		return val[i];
	}
	
	/** 
	* Calculates the distance between this entity and the parameter entity. For 
	* two entities r1 and r2 the distance between them is calculated as the sum 
	* of one term for each of their attributes.
	* For a numerical attribute, its term is:
	* 	= |v1 - v2| / |D|, where v1 and v2 are the values of that attribute in entity 
	* 	r1 and, respectively, r2, and |D| is the size of the domain from where that 
	* 	attribute take values, for all the entities in the dataset.
	* For a categorial attribute, its term is:
	*	= H(first common ancestor of v1 and v2) / H(hierarchy of the attribute), where 
	*	v1 and v2 are the values of that attribute in entity r1 and, respectively, r2, 
	*	and H denotes the function height of a hierarchy or subhierarchy.
	*
	* @param e	the entity to which the distance is calculated from the current entity.
	* @param csAll	the summary of the cluster containing all entities in the dataset to be k-anonymized; 
	*				the definition of distance between two entities uses this summary.
	* @param d	the descriptor this entity and the parameter entity conform to.
	* @return	the distance between this entity and the parameter entity.
	*/
	public double getDist(Entity e, ClusterSummary csAll, Descriptor d) {
		double result = 0;
		for (int i=0; i<d.getNoQuaAttr(); i++) {
			Attribute a = d.getQuaAttribute(i);
			
			switch (a.getType()) {
      			case AttributeType.NUMERIC : 
      				result += java.lang.Math.abs(((Double) getVal(i)).doubleValue() - ((Double) e.getVal(i)).doubleValue()) /
							  (((Interval) csAll.summ[i]).getMax()-((Interval) csAll.summ[i]).getMin());
   			  		break;
			    case AttributeType.CATEGORICAL :
			    	HNode n1 = a.getHierarchy().searchNode((String) getVal(i), a.getHierarchy().getRoot());
			    	HNode n2 = a.getHierarchy().searchNode((String) e.getVal(i), a.getHierarchy().getRoot());
			    	HNode ancestor = a.getHierarchy().findAncestor(n1, n2);
			    	result += (a.getHierarchy().height(ancestor) + Double.parseDouble("0.0"))/
			    			   a.getHierarchy().height(a.getHierarchy().getRoot());
   	    		  	break;
    			}
		}

		return result;
	}
	
	/**
	* Displays to the standard output the current entity: its name followed by the values of each of the attributes.
	*/
	public void out() {
		System.out.print(name + ": ");
		for (int i=0; i<val.length; i++)
			System.out.print(val[i] + ", ");
		System.out.println();
	}	
}
