/*
 Alina Campan - Hierarchy.java
 -----------------------------
 Date Created: 	08-10-2006
 Date Modified:	03-18-2008
 -----------------------------
 
 It contains the following class:
 	
	- Hierarchy - represents an attribute hierarchy
*/



/** 
* Hierarchy represents a tree hierarchy (not necessarilly balanced), defined over 
* the domain of a categorical or numeric attribute.
*
* Hierarchies are currently used ONLY for categorical attributes, although defined for numeric attributea as well.
*
* Example of hierarchy definition for a categorical attribute:
*	a = b, c, d,
*	b = e, f,
*	c = g, h, i,
* which represents the following hierarchy:
*			  a
*          /  |  \
*        b    c    d
*       /\   /|\
*      e  f g h i
*
* Example of hierarchy definition for a numeric attribute:
*	10-90 = 10-40, 40-60, 60-90
*	10-40 = 10-20, 20-40
*	40-60 = 40-50, 50-60
*	60-90 = 60-70, 70-80, 80-90
* which represents the following hierarchy:
*			       10-90
*            /       |     \
*          /         |       \
*     10-40       40-60         60-90
*     /\           / \         /  | \
*   /    \       /     \     /    |   \
* 10-20,20-40 40-50 50-60 60-70 70-80 80-90
*
* A hierarchy is a tree of HNodes.
*
* A hierarchy is associated to an attribute, stored as the data field a.
* 
* For a hierarchy we keep its root, stored as the data field root; starting from the root, every HNode in the hierarchy 
* is accesible.
*
* @author Alina Campan
*/


import java.io.*;
import java.util.*;
public class Hierarchy {
	Attribute a; // the attribute to which the hierarchy belongs
	HNode root; // the root of the hierarchy
	
	/** 
	* Constructs a hierarchy from a file containing its definition and associates it to an attribute. Two examples of 
	* hierarchies definitions are provided above.
	* @param hierarchyFileName	the name of the file containing the hierarchy definition.
	* @param attr	the attributes to which the hierarchy is associated.
	*/	
	Hierarchy(String fileName, Attribute attr) throws Exception {
		
		byte atRoot = 0;
		String line;
		String cname = "";
		
		a = attr;
		int atype = a.getType();
		
	    switch (atype) {
	    	case AttributeType.NUMERIC: cname="NumericHNode";
			break;
	    	case AttributeType.CATEGORICAL: cname="CategoricalHNode";
			break;
    	}

		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
		while ((line = br.readLine() ) != null) {
			StringTokenizer st = new StringTokenizer(line);
			String cdata = st.nextToken();
						
			if (atRoot==0) {
				root = (HNode) (Class.forName(cname)).newInstance();
				root.setData(cdata);
				atRoot = 1;
				// System.out.println("Creating hierarchy root " + cdata);
			}

			createDescendants (line.substring(line.indexOf('=')+1), cdata);
		}
		
		br.close();		
	}
	
	/** 
	* Creates the descendants of the node in the current hierarchy that contains the data indicated by the parameter data.
	* The descendants are indicated in the string s, which has the form "vdesc_1, vdesc_2, ... , vdesc_n". 
	* @param s	a string containing the values of the descendant nodes to be created.
	* @param data	indicates the node for which the descendants are to be created = the node in the current hierarchy that contains data.
	*/		
	public void createDescendants (String s, Comparable data) throws Exception {
		// s = vd1, vd2, ... , vdn
		
		String cname = "";
		
		int atype = a.getType();
	    switch (atype) {
	    	case AttributeType.NUMERIC: cname="NumericHNode";
			break;
	    	case AttributeType.CATEGORICAL: cname="CategoricalHNode";
			break;
    	}

		//System.out.println("Searching for node " + data);
		HNode cn = searchNode(data, root);
		if (cn != null) {
			StringTokenizer st = new StringTokenizer(s, "\t ,");
			cn.desc = new HNode[st.countTokens()];
			int i=0;
			while (st.hasMoreTokens()) {
				cn.desc[i] = (HNode) (Class.forName(cname)).newInstance();
				cn.desc[i].setData(st.nextToken());
				cn.desc[i].setParent(cn);
				//System.out.println("Creating hierarchy node -" + cn.desc[i].toString() +"-");
				i++;
			}
		}
	}

	/** 
	* Recursively searches in the subtree of node n the node that contains the data s.
	* @param s	we search for the node in the current hierarchy that contains the data s.
	* @param n	indicates the node from where the search begins (the subtree of root n is searched for a node containing data s).
	* @return	the node containing data s, if such a node is found in the subtree of node n; otherwise, returns null.
	*/		
	public HNode searchNode (Comparable s, HNode n) {
		HNode result = null;
		
		if (n.searchData(s) == 0) result=n;
		else if (n.desc != null) { 
			int i = 0;
			while ((i<n.desc.length) && (result==null)) {
				result = searchNode(s, n.desc[i]);
				i++;
			}
		}
		return result;
	}
	
	/** 
	* Returns the root of the hierarchy.
	* @return	the root of the hierarchy.
	*/
	public HNode getRoot() {
		return root;
	}
	
	/** 
	* Calculates the depth in the hierarchy where the node n is located (the levels from that node up to the root).
	* @param n	the node for which to calculate the depth in the hierarchy.
	* @return	the depth in the hierarchy where the node n is located.
	*/
	public int depth(HNode n) {
		HNode cn = n;
		int result = 0;
		while (cn.getParent()!=null) {
			result++;
			cn = cn.getParent();
		}
		return result;
	}
	
	/** 
	* Calculates, recursively, the height of the subhierarchy having as root the node n.
	* @param n	the node for which to calculate the height of its subtree (subhierarchy).
	* @return	the height of the subhierarchy having as root the node n.
	*/
	public int height(HNode n) {
		if (n.desc == null) 
			return 0; 
		else {
			int descHeight = -1;
			for (int i=0; i<n.desc.length; i++)
				if (descHeight < height(n.desc[i])+1)
					descHeight = height(n.desc[i]);
			return (descHeight+1);			
		}
	}
	
	/** 
	* Finds the first common ancestor of nodes n1 and n2; in the worst case, this common ancestor will be the root.
	* @param n1 the first node for which we search the common ancestor.
	* @param n2	the second node for which we search the common ancestor.
	* @return	the first common ancestor of nodes.
	*/
	public HNode findAncestor(HNode n1, HNode n2) {
	
		if (depth(n2)>depth(n1)) {
			HNode aux = n1;
			n1 = n2;
			n2 = aux;
		}
		// now n1 has depth >= then n2
		
		int levelsToClimb = depth(n1)-depth(n2);		
		for (int i=0; i<levelsToClimb; i++)
			n1 = n1.getParent();
		// now n1 and n2 are at the same hierarchy level
		
		while (n1!=n2) {
			n1 = n1.getParent();
			n2 = n2.getParent();			
		}
		
		return n1;
	}
	
	/** 
	* Verifies if node n1 is an ancestor for node n2.
	* @param n1	we verify if there exist a ancestor-descendant relationship between n1 and n2.
	* @param n2	we verify if there exist a ancestor-descendant relationship between n1 and n2.
	* @return	returns 0 if n1 is ancestor for n2; otherwise returns 1.
	*/
	public int isAncestor(HNode n1, HNode n2) {
		int result = 1;
		HNode n = n2;
		while ((n!= null) && (result==1)) {
			if (n1==n) result=0;
			else n=n.getParent();
		}
		return result;
	}
	
 }
